@@ -63,7 +63,7 @@ sub apply_ns {# Plugin
 }
 
 sub apply_route {# meth in Plugin
-  my ($self, $r_hash) = @_;
+  my ($self, $r_hash) = @_;# $r_hash - это строка запроса маршрута из БД
   my $r = $self->app->routes;
   
   $self->app->log->debug("Skip disabled route id=[$r_hash->{id}] [$r_hash->{request}]")
@@ -90,7 +90,7 @@ sub apply_route {# meth in Plugin
   #~ $nr->over(authenticated=>$r_hash->{auth});
   # STEP ACCESS
   $nr->over(access => $r_hash);
-  my $host = eval($r_hash->{host_re} || $r_hash->{host});
+  my $host = eval($r_hash->{host_re} || $r_hash->{host} || '');
   $nr->over(host => $host)
     if $host;
   
@@ -397,7 +397,7 @@ Please report any bugs or feature requests at L<https://github.com/mche/Mojolici
 
 =head1 COPYRIGHT
 
-Copyright 2016 Mikhail Che.
+Copyright 2016+ Mikhail Che.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
